@@ -11,11 +11,11 @@ use warnings;
 
 use Scalar::Util;
 use Digest::MD5;
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 our @EXPORT = qw[ wrap_hash ];
 
-our @CARP_NOT = qw( Hash::Wrap );
+our @CARP_NOT = qw( Hash::Wrap::Meta Hash::Wrap::Meta::Void Hash::Wrap::Meta::Value );
 our $DEBUG    = 0;
 
 # copied from Damian Conway's PPR: PerlIdentifier
@@ -111,6 +111,11 @@ sub _autoload {
     _generate_accessor( $hash_class, $class, $key );
 }
 
+sub _is_wrap {
+    my $class = Scalar::Util::blessed( $_[0] ) // $_[0];
+    return defined $class && defined $REGISTRY{$class};
+}
+
 sub _can {
     my ( $self, $key, $CLASS ) = @_;
 
@@ -261,7 +266,7 @@ sub _build_class {
     my $class = $attr->{-class};
 
     return $class if defined $REGISTRY{$class};
-    my $rentry = $REGISTRY{$class} = { methods => {} };
+    my $rentry = $REGISTRY{$class} = { methods => {}, attr => { %$attr}  };
 
     my %closures;
     my %dict = (
@@ -375,7 +380,7 @@ our $predicate_template = q[
    return exists $self->{\<<KEY>>};
   }
 
-  \$Hash::Wrap::REGISTRY{methods}{'has_\<<KEY>>'} = undef;
+  $Hash::Wrap::REGISTRY{methods}{'has_\<<KEY>>'} = undef;
 
   \&has_\<<KEY>>;
 ];
@@ -478,6 +483,7 @@ END
     $rentry->{validate} = _find_symbol( $class, 'validate', [ 'REF', 'CODE' ] );
 
     Scalar::Util::weaken( $rentry->{validate} );
+    Scalar::Util::weaken( $_ ) for grep ref $_, values %{ $rentry->{attr} };
 
     return $class;
 }
@@ -678,7 +684,7 @@ Hash::Wrap - create on-the-fly objects from hashes
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =head1 SYNOPSIS
 
